import java.awt.*;
import gishur.applet.*;
import gishur.gui.*;
import gishur.gui.displayobject.*;
import gishur.gui.inputhandler.*;
import gishur.awt.event.*;
import gishur.x.*;
import gishur.lang.*;

/**
* Applet-Geruest fuer Aufgabe 12 (Schnitt von n Liniensegment mittels Sweep)
*
* Bedienung: Eingabe von Segmenten durch Druecken der linken Maustaste.
* Start des Sweeps durch Druecken der rechten Maustaste.
*
* Die Klasse SegmentIntersectionSweep muss den eigentlichen Sweep-Algorithmus
* enthalten, so dass an dieser Klasse eigentlich nichts geaendert werden muss.
*/
public class SegmentIntersection extends ApplicationApplet implements ObjectListener {

 /** Displaymanager */
 private DisplayManager display;
 /** Layer mit den eingegebenen Segmenten */
 private Layer xlayer;
 /** Layer mit den Schnittpunkten */
 private Layer player;
 /** Die Liste mit den Segmenten */
 private SimpleList segments=new SimpleList();

 /**
 * Main-Methode zum Starten des Applets als Application.
 */
 public static void main(String[] input_args) {
  new SegmentIntersection().initApplication("SegmentIntersection","Segment Intersection Sweep",800,600);
 }

 /**
 * Initialisiert das Applet.
 */
 public void init() {
  // Generieren eines DisplayManagers mit 2 Layers zur Darstellung
  display=new DisplayManager(2);
  // Erzeugen der Layers
  xlayer=new Layer(); player=new Layer();
  // damit beide Layer dieselbe Skalierung haben, sollten sie auch dieselbe
  // Transformation teilen... 
  player.setTransformation(xlayer.getTransformation());
  // die Layer werden nun an den DisplayManager gehangen...
  display.layers().add(player);
  display.layers().add(xlayer);
  // BorderLayout fuer den Applet-Panel aktivieren
  setLayout(new java.awt.BorderLayout());
  // und den DisplayManager anfuegen
  add("Center",display);

  // Erzeuge Inputhandler fuer die Segmenteingabe
  display.inputs().add(new InputXSegment(xlayer));
  // und den Start der Berechnung
  display.inputs().add(new InputMouseButton());
  
  // dieses Applet dient auch als ObjectListener; um alle Events
  // der Inputhandler abzufangen, fuegen wir es als ObjectListener an
  // den DisplayManager an
  display.addObjectListener(this);
 }


 /**
 * Ein Inputhandler hat ein ObjectEvent verschickt.
 * @param e ObjectEvent mit Daten (und Herkunft)
 */
 public void actionPerformed(ObjectEvent e) {
  switch (e.getSubType()) {
   // ein Segment wurde eingegeben
   case ObjectEvent.INPUT_OBJECT:
    // Segment an die Liste anfuegen
    segments.add(e.getTarget());
    // und mittels DisplayXSegment an den xlayer fuegen
    // und somit darstellen
    xlayer.add(new DisplayXSegment((XSegment)e.getTarget()));
    break;
   // die rechte Maustaste wurde gedrueckt
   case ObjectEvent.DO_ACTION:
    System.out.println("Segmentliste:  "+segments);
    // Sweep ausfuehren (Ergebnis ist eine Punkteliste)
    SimpleList points=new SegmentIntersectionSweep().segmentIntersection(segments);
    System.out.println("Schnittpunkte: "+points);
    // Punkte an den player fuegen und somit darstellen
    player.clear();
    while(!points.empty())
     player.add(new DisplayXPoint((XPoint)points.pop()));
    break;
  }
 }

 // Die folgenden Methoden muessen nur wegen ObjectListener implementiert werden:

 /**
 * Wird aufgerufen, bevor eine Modifikation an einem Objekt begonnen hat.
 * @param e ObjectEvent mit den Daten
 */
 public void startModification(ObjectEvent e) {}
 /**
 * Wird aufgerufen, whrend eine Modifikation an einem Objekt stattfindet.
 * @param e ObjectEvent mit den Daten
 */
 public void doModification(ObjectEvent e) {}
 /**
 * Wird aufgerufen, wenn eine Modifikation erfolgreich beendet wurde.
 * @param e ObjectEvent mit den Daten
 */
 public void finishModification(ObjectEvent e) {}
 /**
 * Wird aufgerufen, wenn eine Modifikation abgebrochen wurde.
 * @param e ObjectEvent mit den Daten
 */
 public void cancelModification(ObjectEvent e) {}
}
