import gishur.x.*;
import gishur.lang.*;
import java.util.Enumeration;

public class SegmentIntersectionSweep extends Sweep {
   
   // Ereignis-IDs
   
   /** Ereignis-ID: linker Endpunkt erreicht */
   public final static byte LEFT_ENDPOINT=1;
   /** Ereignis-ID: vertikales Segment erreicht */
   public final static byte VERTICAL=2;
   /** Ereignis-ID: Schnittpunkt erreicht */
   public final static byte INTERSECTION=3;
   /** Ereignis-ID: Schnittpunkt erreicht */
   public final static byte INTERSECTION_POINT=4;
   /** Ereignis-ID: rechter Endpunkt erreicht */
   public final static byte RIGHT_ENDPOINT=5;
   
   /** Liste mit den Ausgabepunkten. */
   private SimpleList      _points;
   /** Linienvergleicher fr die SSS */
   private LineComparitor  _linecomparitor;
   /** letzter erreichter Schnittpunkt */
   private XPoint          _lastcut=null;
   
   /**
   * Fhrt einen Plane-Sweep durch, um alle Schnittpunkte der in der Liste L bergebenen
   * Segmente zu bestimmen. Die Ausgabeliste enthlt alle Schnittpunkte sortiert nach
   * X-Koordinaten (PointComparitor.X_ORDER).<br>
   * @param L Segmentliste (Liste mit XSegment-Objekten)
   * @return Liste mit den Schnittpunkten
   */
   public SimpleList segmentIntersection(SimpleList L) {
      if ((L==null) || (L.length()<2)) return new SimpleList();
      createEventStructure(new SweepEventComparitor(
         new PointComparitor(PointComparitor.X_ORDER)),true);
      _linecomparitor=new LineComparitor();
      createSSS(_linecomparitor);
      XSegment s;
      for (ListItem i=L.first();i!=null;i=i.next()) {
         if (i.key() instanceof XSegment) {
            s=(XSegment)i.key();
            if (s.vertical()) {
               insertEvent(VERTICAL,s.getStartPoint(),s);
            } else {
               insertEvent(LEFT_ENDPOINT,s.getStartPoint(),s);
               insertEvent(RIGHT_ENDPOINT,s.getEndPoint(),s);
            }
         }
      }
      _points=new SimpleList(); _lastcut=null;
      execute();
      return _points;
   }
   
   /**
   * Verarbeitet das SweepEvent e.
   * @param e SweepEvent
   */
   public void processEvent(SweepEvent e) {
      XPoint p=(XPoint)e.key();
      _linecomparitor.setX(p.x);
      TreeItem i,j,k;
      XSegment[] sl;
      switch (e.getID()) {
         // Linker Endpunkt
      case LEFT_ENDPOINT:
         _linecomparitor.setDelta(LineComparitor.BEFORE);
         i=sss().add(e.value());
         testIntersection(i,true);
         testIntersection(i,false);
         break;
         // Rechter Endpunkt
      case RIGHT_ENDPOINT:
         _linecomparitor.setDelta(LineComparitor.AFTER);
         i=sss().find(e.value()); j=sss().prev(i);
         sss().remove(i);
         testIntersection(j,true);
         break;
         // vertikales Segment
      case VERTICAL:
         _linecomparitor.setDelta(LineComparitor.EXACT);
         XPoint q=((XSegment)e.value()).getEndPoint();
         XSegment v=new XSegment(p,p), w=new XSegment(q,q);
         i=sss().find(v); j=sss().find(w);
         if (i==null) i=sss().findBigger(v);
         if (j==null) j=sss().findSmaller(w);
         if ((j!=null) && (sss().next(j)==i)) i=null;
         if (j!=null) j=sss().next(j);
         while ((i!=null) && (i!=j)) {
            sl=new XSegment[2];
            sl[0]=(XSegment)e.value(); sl[1]=(XSegment)i.key();
            insertEvent(INTERSECTION_POINT,sl[1].calculatePoint(p.x),sl);
            i=sss().next(i);
         }
         break;
         // (normaler) Schnittpunkt
      case INTERSECTION:
         sl=(XSegment[])e.value();
         _linecomparitor.setDelta(LineComparitor.BEFORE);
         // suche das bislang unter Segment j und das obere k
         j=sss().find(sl[0]); k=sss().find(sl[1]);
         // wenn derselbe Schnitt schoneinmal berechnet wurde, dann raus
         if ((j==null) || (k==null)) break;
         // Vertausche oberes und unteres Segment...
         sss().remove(j); sss().remove(k);
         _linecomparitor.setDelta(LineComparitor.AFTER);
         j=sss().add(j); k=sss().add(k);
         testIntersection(j,true);
         testIntersection(k,false);
         // weiter mit INTERSECTION_POINT !!!
         // entspricht: insertEvent(INTERSECTION_POINT,p,seg);
         // Berichten von (normalen und vertikalen) Schnittpunkten
      case INTERSECTION_POINT:
         if ((_lastcut==null) || (!_lastcut.equals(p))) _points.add(p);
         _lastcut=p;
         break;
      }
   }
   /**
   * Testet den Schnitt des Segments im TreeItem t der SSS mit dem darberliegenden - falls
   * up==true, ansonsten mit den darunterliegenden - Segment.
   * Falls ein Schnitt entdeckt wurde, wird ein entsprechendes Schnittevent eingefgt
   * @param t TreeItem der SSS
   * @param up falls true, Test mit darberliegenden (nchsten) Segment, ansonsten mit dem
   * darunterliegenden (vorigen) Segment
   */
   private void testIntersection(TreeItem t,boolean up) {
      // Knoten i1 und i2 mit i1<i2 entsprechend t und up bestimmen
      TreeItem i1=(up)?t:sss().prev(t);
      TreeItem i2=(up)?sss().next(t):t;
      // Kein Schnittest ntig, falls einer der Knoten null
      if ((i1==null) || (i2==null)) return;
      // Schnitt der gespeicherten Segmente bestimmen
      XSegment s1=(XSegment)i1.key(),s2=(XSegment)i2.key();
      Intersection inter=s1.intersection(s2);
      // Kein Schnittpunkt -> raus
      if (inter.xpoint()==null) return;
      // INTERSECTION-Event generieren
      XSegment[] sl={s1,s2};
      insertEvent(INTERSECTION,inter.xpoint(),sl);
   }
   
}